#    Amazing Text Fun Game Engine - a engine for the decoding and play of .atf game files
#    Copyright (C) <2002>  <Kenneth Bromberg, Antoni Chan>
#    See README for full license information

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Questions or comments?  Contact the author - atfengine@alltooflat.com

use Cwd;
require Exporter;
@ISA = qw(Exporter);
@Export = qw(IsGameDir, FindGames, GetGameCode, GetGameName, GetGameDescript,_
			 GetGameMainFileName, GetNumRooms, LoadItems, LoadItemHash, LoadRooms);


# ISGAMEDIR($gamedir) - Returns true if the given directory contains a gamefile for an atf text adventure
#
#	$gamedir is a scalar string value holding the name of a directory.
sub IsGameDir
{
	$dirtocheck = shift;
	opendir DIRTOCHECK, $dirtocheck;

	@dirlistcheck = readdir(DIRTOCHECK);
	foreach $direct (@dirlistcheck) 
	{
		if ($direct =~ /game.atf$/)
		{
			return 1;
		}
	}
	closedir DIRTOCHECK;
	return 0;
}

# FINDGAMES() - Examines all directories in working directory, and returns a list
#	of directories (scalars) in which atf text games are found
#
#	Takes no arguments, and returns a list of valid game directories
#	TODO:  Make it so that, with the scalar(@_) command, we can specify a directory in which to check for games
sub FindGames
{

	my @dirlist;

	# If no arguments have been given to FindGames, the directory to search is the working directory
	if (scalar(@_) eq 0) 
	{
		$dir = cwd();
		opendir PROGDIR, $dir;
		@dirlist = readdir(PROGDIR);
	}
	elsif (scalar(@_) eq 1)
	{
		$tdir = undef;
		$dir = shift;
		my @temp = split(/\//, $dir);
		$pathdepth = scalar(@temp);
		opendir PROGDIR, $dir;
		@dirlist = readdir(PROGDIR);
	}

	

	# Next block goes through all the directories in the current operating directory, 
	#	determining which, if any, directories contain an atf text game.  Puts the 
	#	directory string into @GAMELIST
	my $i = 0;
	foreach $entry (@dirlist) 
	{
		$tempdir = $dir . "/" . @dirlist[$i];
		if (opendir TEMPDIR, $tempdir)
		{
			if (IsGameDir($tempdir))
			{
				push @gamelist, $tempdir;
			}
			closedir TEMPDIR;
		}
		$i++;
	}
	closedir PROGDIR;
	if (scalar(@gamelist) eq 0)		# If no games were found in the directory
	{
		push @gamelist, 0;
	}
	return @gamelist;
}


# GETGAMECODE($mainfilepath) - Opens the game file in specified directory, and returns the integer gamecode
#
#	Takes the path/gamefilename of the main game file:  Generated from GETGAMEMAINFILENAME()
sub GetGameCode
{
	$mainfilepath = shift;

	open MAINGAMEFILE, "<$mainfilepath" or die "Gamefile could not be opened!  Failure: $!\n";
	for (my $i=1;$i<=$mainline_GAMECODE ;$i++) 
	{
		chop($gamecode = <MAINGAMEFILE>);
	}
	close MAINGAMEFILE;
	return $gamecode;
}


# GETGAMENAME($mainfilepath) - Opens the game file in specified directory, and returns the name of the game
#
#	Takes the path/gamefilename of the main game file:  Generated from GETGAMEMAINFILENAME()
sub GetGameName
{
	$mainfilepath = shift;

	open MAINGAMEFILE, "<$mainfilepath" or die "Gamefile could not be opened!  Failure: $!\n";
	for (my $i=1;$i<=$mainline_GAMENAME ;$i++) 
	{
		chop($gamename = <MAINGAMEFILE>);
	}
	
	close MAINGAMEFILE;
	return $gamename;
}

# GETGAMEDESCRIPT($gamedir) - Opens the game file in specified directory, and returns the name of the game
#
#	Takes the path/gamefilename of the main game file:  Generated from GETGAMEMAINFILENAME()
sub GetGameDescript
{
	$mainfilepath = shift;

	my $gamedesc;

	open MAINGAMEFILE, "<$mainfilepath" or die "Gamefile could not be opened!  Failure: $!\n";
	for (my $i=1;$i<=$mainline_DESCRIPT; $i++) {
		chop($gamedesc = <MAINGAMEFILE>);
	}
	close MAINGAMEFILE;
	return $gamedesc;
}

# GETGAMEMAINFILENAME($directory) - Returns path/filename of the main game file in the specified directory.
#
#	e.g.  GetGameMainFileName($somedir) might return "c:/Games/AllTooFlat/ATFGameEngine/Zork23/zork23game.atf"
sub GetGameMainFileName
{
	$gamedir = shift;
	opendir GAMEDIR, $gamedir;
	@dirlist = readdir(GAMEDIR);
	foreach $entry (@dirlist) 
	{
		if($entry =~ /game.atf$/)
		{
			return $gamedir.'/'.$entry;
		}		
	}
	closedir GAMEDIR;
}


#	GETNUMROOMS($gametoplay) - Returns the number of areas in the game, as defined by the main game file (eg zorkgame.atf)
sub GetNumRooms
{
	$mainfilepath = shift;
	my $numrooms;

	open MAINFILE, $mainfilepath;
	for (my $i=1; $i<=$mainline_NUMROOMS; $i++) 
	{
		$numrooms = <MAINFILE>;
	}
	return $numrooms;
	close MAINFILE;
}


# LOADITEMS
#	This sub routine goes through MAINFILE (XXXXroom.atf) starting at the beginning and sends the item descriptions
#	to the itemlist objects line by line (where they are there decoded.)
sub LoadItems
{
	my $i =0;
	@ilist;
	open MAINFILE, $basepath."room.atf" or die "I'm afraid that didn't work so well: $_\n";

	$newline = <MAINFILE>;

	# Goes through MAINFILE until we get to the beginning of the item list
	until ( $newline eq "BEGINITEMLIST") 
	{
		chop($newline = <MAINFILE>);	
	}

	chop($newline = <MAINFILE>);

	while (!($newline =~ /ENDITEMLIST/)) 
	{
		$ilist[$i] = Item->new;
		$ilist[$i]->ReadItem($newline);
		chop($newline=<MAINFILE>);
		$i++;
	}
	close MAINFILE;
	return @ilist;
}	# End LoadItems sub


# LOADITEMHASH(@ilist) - Steps through the item object list and sets up the item hash for indexing
sub LoadItemHash
{
	$ilist = shift;
	%ihash;

	my $i = 0;
	my $maxitem = scalar(@ilist);
	for ($i;$i<$maxitem ;$i++) 
	{
		my $temp = $ilist[$i]->{ItemName};
		$ihash{$temp} = $i;
	}

	return %ihash;
}


# LOADROOMS
#	This sub goes through the rest of MAINFILE (remaining from after the LOADITEMS sub) and returns the room descriptions in
#	a list of Room objects (which we then return to the LOADGAME sub, which then returns it to the GAMEINIT sub of maingame.pl
sub LoadRooms
{

	my $i = 0;
	@rlist;
	open MAINFILE, $basepath."room.atf" or die "I'm afraid that didn't work so well: $_\n";

	$newline = <MAINFILE>;
	

	until ( $newline eq "BEGINROOMLIST") 
	{
		chop($newline = <MAINFILE>);	
	}

	chop($newline = <MAINFILE>);


	while(!($newline =~/ENDROOMLIS/))
	{
		$rlist[$i] = Room->new;
		$rlist[$i]->ReadRoom($newline);
		chop($newline=<MAINFILE>);
		$i++;
	}
	close MAINFILE;
	return @rlist
}	# End LoadRooms sub

sub LoadActionHash
{

	my $bpass = shift;		# Take the basepath off the arguments list

	%ActionHash;
	$progpath = cwd();		# Is the path where the game files / executable is kept (and therefore the standard actions)

#	open SACTIONFILE, $progpath."/sactions.atf" or die "Standard action list could not be opened.\n";
#
#	chop($newline=<SACTIONFILE>);
#	until ($newline eq "ENDACTIONLIST") 
#	{
#		@ActionToInsert = split(/:/, $newline);
#		$ActionHash{$ActionToInsert[0]} = $ActionToInsert[1];	# Inserts an actionword / switchcode pair into the hash
#		chop($newline=<SACTIONFILE>);
#	}
#
#	close SACTIONFILE;

	$ExtraActionFlag = 1;

	open EACTIONFILE, $basepath."eactions.atf" or $ExtraActionFlag = 0;
	if ($ExtraActionFlag) 
	{
		chop($newline=<EACTIONFILE>);
		until ($newline eq "ENDACTIONLIST") 
		{
			@ActionToInsert = split(/:/, $newline);
			$ActionHash{$ActionToInsert[0]} = $ActionToInsert[1];	# Inserts an actionword / switchcode pair into the hash
			chop($newline=<EACTIONFILE>);
		}
	}
	return %ActionHash;
}

sub LoadComments
{
	my $bpass = shift;
	my $i = 0;
	my @clist;
	open MAINFILE, $bpass."room.atf" or die "I'm afraid that didn't work so well: $_\n";

	$newline = <MAINFILE>;

	until ( $newline eq "BEGINCOMMENTLIST") 
	{
		chop($newline = <MAINFILE>);
	}

	chop($newline = <MAINFILE>);

	while (!($newline=~/ENDCOMMENTLI/)) 
	{
		my @thang = split(/\^/, $newline);
		$clist[$i] = $thang[1];
		chop($newline=<MAINFILE>);
		$i++;
	}

	close MAINFILE;
	return @clist;
}