#    Amazing Text Fun Game Engine - a engine for the decoding and play of .atf game files
#    Copyright (C) <2002>  <Kenneth Bromberg, Antoni Chan>
#    See README for full license information

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Questions or comments?  Contact the author - atfengine@alltooflat.com


# There are a lot of things that are going to have to be defined as "items":  Anything that has an effect on the state of
#	any room in the game (state changes) has to have switch codes associated with it, and therefore must be an item
#

package Item;

sub new
{
	my $Class = shift;
	my $self = {};

	$self->{NumStates} = undef;
	$self->{ItemName} = undef;		# A items can only have a single name
	$self->{ItemWeight} = undef;	# A list of item weights, corresponding to the weight of an item in different states
	$self->{ItemDesc} = undef;		# A list of item descriptions, corresponding to the weight of an item in different states
	$self->{SwitchCodes} = undef;	# A list of number codes, seperated by a special character (&).  These can be decoded using 
									#  one of the methods in the Action package.  this is the set of SwitchCodes FOR THE CURRENT
									#  ACTIVESTATE OF THE ITEM!!!!! VERY Important.

	$self->{ActiveState} = 0;		# A number which corresponds to the active state of the item.  Defaults to zero, the first
									# item state
	
	$self->{SwitchCodeList} = undef;	# A list switchcode lines, any of which can be sent to ASSIGNACTIVESWITCHCODES for decoding
										# and insertion into $self->{SwitchCodes}
	

	$self->{ShowInInv} = undef;		# Do we want the item to be listed in an examination of the room and its objects?
									# 1 if yes, 0 if no
									# It's a LIST, each entry corresponding to an item state

	$self->{CanBeTaken} = undef;	# Can the item be picked up and put in inventory?
									# It's a LIST, each entry corresponding to an item state


	bless $self, $Class;
}


# Takes in a single line of an item definition, decodes it, and sets the appropriate values of the object
#
#	TODO:  Set up item definitions to reflect binary switches (eg ShowInInc, etc)
sub ReadItem
{
	my $self = shift;
	my $itemline = shift;

	my $itemline_NUMSTATES = 1;
	my $itemline_NAME = 2;
	my $itemline_1STDESC = 3;
	my $itemline_1STWEIGHT = 4;
	my $itemline_1STSWITCHCODES = 5;
	my $itemline_1STSHOWININVBIT = 6;
	my $itemline_1STCANBETAKENBIT = 7;		#Make sure that the CANBETAKEN bit is always the last bit in an item description

	my $NumThingsInStateDescription = $itemline_1STCANBETAKENBIT - $itemline_1STDESC + 1;

	my $scodes;

	@temp = split(/\^/, $itemline);

	$self->{NumStates} = $temp[$itemline_NUMSTATES]; 
	$self->{ItemName} = $temp[$itemline_NAME];

	for ($index=0; $index<$self->{NumStates} ; $index++) 
	{
		$self->{ItemDesc}[$index] = $temp[$itemline_1STDESC + $index*$NumThingsInStateDescription];
		$self->{ItemWeight}[$index] = $temp[$itemline_1STWEIGHT + $index*$NumThingsInStateDescription];
		$self->{SwitchCodeList}[$index] = $temp[$itemline_1STSWITCHCODES + $index*$NumThingsInStateDescription];
		$self->{ShowInInv}[$index] = $temp[$itemline_1STSHOWININVBIT + $index*$NumThingsInStateDescription];
		$self->{CanBeTaken}[$index] = $temp[$itemline_1STCANBETAKENBIT + $index*$NumThingsInStateDescription];
	}
	
	my $scodes = $temp[$itemline_1STSWITCHCODES];

	&AssignActiveSwitchCodes($self, $NewState);
}



#	ASSIGNACTIVESWITCHCODES($self, $NewState) - Takes in the value of the new state of the item, extracts the SwitchCodes from
#												the SwitchCodeList of the Item, and inserts it into {SwitchCodes} with the 
#												proper decoding
#
#	NB:  When calling this method from another method in Item which has the "$self = shift;" line, we have to supply $self as  
#	an argument.
sub AssignActiveSwitchCodes
{
	my $self = shift;
	my $NewState = shift;
	
	my $scodes = $self->{SwitchCodeList}[$NewState];
	
	my @scodelist = split(/\|/, $scodes);			#	Cuts up the switchcodes on vertical bars

	my $tempcode;	

	foreach $tempcode (@scodelist) 
	{
		my @tcode = split(/->/, $tempcode);
		$self->{SwitchCodes}{$tcode[0]} = $tcode[1];
	}
}	# End ASSIGNACTIVESWITCHCODES($)


#	CHANGEACTIVESTATE($NewState) - Changes all the parameters in the object to set the item to a new state
#
#	Takes as an argument the desired item state
sub ChangeActiveState
{
	my $self = shift;
	my $NewState = shift;

	&AssignActiveSwitchCodes($self, $NewState);
	$self->{ActiveState} = $NewState;
}	# End AssignActiveSwitchCodes($)



# GETDESC() - Returns a scalar description of the item for the items active state
sub GetDesc
{
	$self = shift;
	return $self->{ItemDesc}[$self->{ActiveState}];
}


# GETSHOWININV() - Returns the ShowInInv bit of the item for the items active state
sub GetShowInInv
{
	$self = shift;
	return $self->{ShowInInv}[$self->{ActiveState}];
}



# GETCANBETAKEN() - Returns the CanBeTaken bit of the item for the items active state
sub GetCanBeTaken
{
	$self = shift;
	return $self->{CanBeTaken}[$self->{ActiveState}];
}


# GETITEMWEIGHT() - Returns the CanBeTaken bit of the item for the items active state
sub GetItemWeight
{
	$self = shift;
	return $self->{ItemWeight}[$self->{ActiveState}];
}


# EXECUTESWITCH($switchcode) - Where the magic of cinema occurs
sub ExecuteSwitch
{
	my $self = shift;
	my $SwitchCode = shift;


	if ($self->{SwitchCodes}{$SwitchCode})		# That's IF the switchcode sent to the item is a recognized switchcode 
												#	in the description of the item
	{
		my @ActionResultList = split(/&/, $self->{SwitchCodes}{$SwitchCode});	# ACTIONRESULTLIST is a list of actions 
																				# that should be undertaken when a 
																				# particular switchcode is activated in an item
																				# These codes should correspond to VerbCodes that
																				# can themselves be decoded in the ACTION package

		foreach $action (@ActionResultList) 
		{
			if ( !(Main::ExecuteVerbCodeI($action)))	# If we don't get a true back from ExecuteVerbCodeI, the process stops
			{
				print "Nothing happened.\n";
				last;

			}
		}

	return 1;	# If the action was a valid switchcode appearing in the switchcode list of the item, we'll return 1.  Else we 
				# return zero.  We may or may not use this information after we call ExecuteSwitch.
	}
	else {
		return 0;
		}


	
}
1;