#    Amazing Text Fun Game Engine - a engine for the decoding and play of .atf game files
#    Copyright (C) <2002>  <Kenneth Bromberg, Antoni Chan>
#    See README for full license information

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Questions or comments?  Contact the author - atfengine@alltooflat.com


#  This main package will contain all the methods and variables needed for the front end of the game engine
#
#	TODO:  Set up a system that will allow the game designer to specify his own commands / verbs and switchcode triggers
#					(I don't think you'll be able to specify switchcode results, however...

package Main;

use Person;
use Room;
use Item;
use GameAdmin;
use Action;
use Abcutil;

############################### Begin GameInit Sequence ##############################
# GAMEINIT($gamefilepath) - Loads the game specified by $gamefilepath into main memory.
sub GameInit
{
	my $gametoplay = shift;

	if ($gametoplay eq 0)		# This is the "Exit from game choice" selection
	{
		die "Goodbye!\n";
	}

	# Removes "main.atf" from the gamename, yielding a basepath
	$basepath = $gametoplay;
	$basepath =~ s/game.atf//; 

	$gamecode = GetGameCode($gametoplay);		# ATF GameCode
	$gamename = GetGameName($gametoplay);		# Game title
	$numrooms = GetNumRooms($gametoplay);		# Number of rooms/areas in game

	
	@AllRooms = &LoadRooms($basepath);
	@ItemList = &LoadItems($basepath);
	@CommentList = &LoadComments($basepath);
	%ItemtoNumHash = &LoadItemHash(\@ItemList);	# To pass a list as a parameter, it must be passed as a reference.
											# Pick it up as a scalar in the subroutine.

	%ActiontoSwitchHash = &LoadActionHash($basepath);

	$ActiveRoom = 0;

	$Player = Person->new();					# Instantiate the player character

}
############################### End GameInit Sequence ##############################



################################ Begin GameSelect  #################################
# GAMESELECT() - Presents the user with a selection of games available for play, and allows the user
#	to select a game.
#
#	Returns a path/filename for the game selected by the user.
sub GameSelect
{
	my @gamelist = FindGames();
	print "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nPlease select a game to load.\n\n";
	$gameshash{"QUIT"} = 0;		# Escape selection for our game


	# Show the list of games, and ask for a selection, repeat as needed
	ShowGameList();
	$outstr = "\nPlease Enter the letter corresponding to the game you'd like to load, type quit to exit.\n>> ";
	print &FormatString($outstr);
	chop($selection = <STDIN>);

		$selection =~ tr/[a-z]/[A-Z]/; 		# Replaces all lowercase letters to uppercase for matching purposes.
		while (!exists $gameshash{$selection}) 
		{
			&ShowGameList();
			$outstr =  "\nI'm afraid you have chosen an invalid option.  Please try again.\n>>";
			print &FormatString($outstr);
			chop($selection = <STDIN>);
			$selection =~ tr/[a-z]/[A-Z]/; 		# Replaces all lowercase letters to uppercase for matching purposes.
		}


		return $gameshash{$selection};



	sub ShowGameList
	{
		$i = 'A';
		foreach $entry (@gamelist) 
		{
			$gfilename = GetGameMainFileName($entry);
			print $i, '.  ', GetGameName($gfilename), '- ', GetGameDescript($gfilename), "\n";
			$gameshash{$i} = $gfilename;	#$GAMESHASH associates the letter selection with the game mainfile name
			$i++;

		}
	} #End Sub ShowGameList
}
############################### End GameSelect Sequence ##############################
 


{
	# Initialize Constants
	$mainline_GAMECODE = 1;
	$mainline_GAMENAME = 2;
	$mainline_DESCRIPT = 3;
	$mainline_NUMROOMS = 4;

	# Allocate memory for variables
	$gametoplay;	# the path/name of the main game file
	$basepath;		# the path/gamename for the game (eg: c:/atfgame/bangzork/zork)
	$numrooms;		# the number of areas in the game
	$gamename;		# the title of the game (eg Zork)
	$gamecode;		# An integer code to make sure XXXXroom.atf and XXXXgame.atf are for same game

	$lastcommand = undef;	# The last command that was input to the game.  For use with the "again" action
	$numlinesprinted = 0;

	$ActiveRoom;	# The number of the room the adventurer is currently in


	%ItemtoNumHash;	# A mapping from ItemNames (all lowercase) to item indexes (in ItemList).  Item indexes are the same 
					# the "Item reference numbers" which precede item definitions in XXXXroom.atf
	

	@ItemList;		# A static list of Item objects:  An items index in this list can be determined by using the ItemName 
					# as a key in ItemHash

	@AllRooms;		# A list of Room objects
	
	@CommentList;	# A list of incidental text strings that can be accessed and printed via item switchcodes

	$Player;		# The player character


	# Code begins here.
	$gametoplay = &GameSelect();

	&GameInit($gametoplay);



	print "\n\n\n";
#
#	@ken = keys(%ActiontoSwitchHash);
#	@ken2 = values(%ActiontoSwitchHash);
#	for ($i=0;$i<scalar(@ken); $i++) 
#	{
#		print "$i:  $ken[$i] - $ActiontoSwitchHash{$ken[$i]} \n";
#	}
#
#
#	if ($temp = $ActiontoSwitchHash{'klll'}) 
#	{
#		print "found it: $temp\n";
#	}
#	else {print "Nope'rs:  $temp\n";}


do {

	&LookAtRoom();
	print ">> ";
	$numlinesprinted = 0;
	$cline = <STDIN>;
	print "\n";
	&ParseCommand($cline);
	print "\n\n";
	} until (0);

}	# End Main