#    Amazing Text Fun Game Engine - a engine for the decoding and play of .atf game files
#    Copyright (C) <2002>  <Kenneth Bromberg, Antoni Chan>
#    See README for full license information

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Questions or comments?  Contact the author - atfengine@alltooflat.com

package Room;
use Cwd;

sub new 
{
	my $Class = shift;
	my $self = {};

	$self->{North};				# Each of these directions will be arrays, each entry of which corresponds to the 
	$self->{South};				#	location you go to for a given state.  e.g.
	$self->{East};				#	$self->{North}[3] = 34; means that when that Room has ActiveState = 3, 
	$self->{West};				#	a game movement north will lead the player to room number 34.
	$self->{Up};
	$self->{Down};

	
	$self->{Description};		# Also a list of descriptions, each entry corresponding to a different state of the room.
	$self->{BeenSeen} = 0;		# Tells whether the player has been in / seen a description of the room before
	$self->{ShowRoomName} = 1;	# Tells whether we should print the room name and inventory of a room at each command cycle
	$self->{RoomName};
	$self->{ActiveState} = 0;	# Each room must have at least one "state," the first one will always be 0
								#	This can be changed by using objects:  Changes are affected through using items
								#	and decoding the switchcodes associated with those items.

	$self->{Inv} = [0];			# A list of item reference numbers, listing the items in a room.
	$self->{NumStates};

	bless $self, $Class;
}


# READROOM($rlist): Takes in and decodes a room description line from xxxxroom.atf
# 
# Kind of still needs to be written.
sub ReadRoom
{
	my $self = shift;
	my $newline = shift;

	# Declare static variables: Esp. positions of entries in a rooms data line
	my $roomline_NUMSTATES = 0;		# Position of the number of states for an area
	my $roomline_NAME = 1;
	my $roomline_INITINV= 2;
	my $roomline_1STDESC = 3;
	my $roomline_1STDIR = 4;

	my @roomline = split(/\^/, $newline); 
	$self->{NumStates} = $roomline[$roomline_NUMSTATES];

	$self->{RoomName} = $roomline[$roomline_NAME];


	# Each room has a number of states associated with it.  This loop is executed $SELF->{NUMSTATES} times, and puts information
	#	for each state into an array entry for the various Room attributes (e.g. Descriptions for each state)
	for ($index=0; $index<$self->{NumStates}; $index++ ) 
	{
		$self->{Description}[$index] = $roomline[$roomline_1STDESC+(2*$index)];
		my @temp = split(/\./, $roomline[$roomline_1STDIR+(2*$index)] );

		$self->{North}[$index] = $temp[0];
		$self->{South}[$index] = $temp[1];
		$self->{East}[$index] = $temp[2];
		$self->{West}[$index] = $temp[3];
		$self->{Up}[$index] = $temp[4];
		$self->{Down}[$index] = $temp[5];
	}

	# Each room may have some initial inventory items associated therewith.  Next block of code loads those items
	#	into each rooms inventory list.  If there is no item in a room initially, a -1 appears in the room definition.
	my @itemlist = split(/\./, $roomline[$roomline_INITINV]);
	foreach $thang (@itemlist) 
	{
		if ($thang eq '-1') 
		{
			last;
		}

		$self->AddToInv($thang);
	}

}

# GETDESC() returns the scalar description of the ActiveState of the room it is called on
sub GetDesc
{
	my $self = shift;

	$self->{BeenSeen} = 1;		# Next time the room is entered, the description will not be shown 
								#	unless there has been a state change
	return $self->{Description}[$self->{ActiveState}];

}

# SETSTATE($NEWSTATE) changes the state of the room that it is called on to the numerical value of the argument
#	Kills the program and prints error statement if it's asked to set a state not in the range of the room.
sub SetState
{
	my $self = shift;
	my $newstate = shift;

	$self->{BeenSeen} = 0;		# Next time the room is entered, the description will appear.
	if( ($newstate < 0) or ($newstate > $self->{NumStates}))
	{
		die "Error:  Attempt to set the state of room ", $self->{RoomName}, " to a value outside the valid range\n";
	}
	$self->{ActiveState} = $newstate;
}

# ADDTOINV($NEWITEM) - Adds a new item to the inventory list for a room.
sub AddToInv
{
	my $self = shift;
	my $toadd = shift;

	$self->{Inv}[0]++;
	$self->{Inv}[$self->{Inv}[0]] = $toadd;
}


# REMOVEFROMINV($ITEMTOREM) - Takes an item out of a rooms inventory
sub RemoveFromInv
{

	my $self = shift;

	my $ItemtoRem = shift;
	if ($itemplace = $self->CheckIfItem($ItemtoRem)) 
	{
			$self->{Inv}[$itemplace] = $self->{Inv}[$self->{Inv}[0]];
			$self->{Inv}[0]--;
			return 1;
	}
	else 
	{ 
		print "\nItem is not in inventory.\n";
		return 0;
	}
}

# CHECKIFITEM($ITEMTOCHECK) -	Returns false if the input item reference number does not exist in room inventory
#								Returns the index of the item in the room inventory if exists in inventory
#
#	Takes as an argument an item reference number
#	Returns 0 if the item is not in the inventory.
#	Returns the index of the item in the inventory if it is in inventory
sub CheckIfItem
{
	my $self = shift;
	my $ItemtoRem = shift;

	for ($i = 1; $i <= $self->{Inv}[0] ; $i++) 
	{
		if ($self->{Inv}[$i] eq $ItemtoRem) 
		{
			return $i;
		}
	}
	return 0;
}

#	GETITEMLIST() - Returns the inventory list of the room.  Could be an empty inventory
#
#	Recall the the rooms inventory list is a list of item reference numbers
sub GetItemList
{
	my $self = shift;
	my @returnlist;
	my $ind;

	for ($ind=1;$ind<=$self->{Inv}[0] ;$ind++) 
	{
		push(@returnlist, $self->{Inv}[$ind]);
	}

	return @returnlist;
}

#  GETINVNAMES() - Returns a list of items the player character is currently carrying (the item names, not ref numbers)
#
#	Mostly for use by the action package and the "All" switch on drop commands
sub GetInvNames
{
	my $self = shift;
	my $thang;
	my @returnlist;

	for ($thang = 1; $thang <= $self->{Inv}[0]; $thang++) 
	{
		my $actualitem = $self->{Inv}[$thang];
		push(@returnlist, $Main::ItemList[$actualitem]->{ItemName});
	}

	return @returnlist;
}	# End GETINVNAMES

# SHOWINV() - Prints a serial list of the items in a room (list of reference numbers on a single line seperated by spaces)
sub ShowInv
{
	my $self = shift;

	if ($self->{Inv}[0] eq 0) {
		print "You have no items.\n";
	}

	for ($index = 1; $index <= $self->{Inv}[0]; $index++) {
		print $self->{Inv}[$index], " ";
	}
	print "\n";
}

#	This function returns a scalar value:  -1 if the room holds no items
#											a list of item numbers seperated by "-"'s if the player is holding items.
sub ReturnItemVector
{
	my $self = shift;

	$ScalarToReturn = "";
	if ($self->{Inv}[0] eq 0) 
	{
		$ScalarToReturn = "-1";
	}
	else
	{
		for ($index = 1; $index <= $self->{Inv}[0]; $index++) 
		{
			$ScalarToReturn = $self->{Inv}[$index]."-".$ScalarToReturn;
		}
		chop($ScalarToReturn);
	}

	return $ScalarToReturn;
}


sub DESTROY
{

}

1;